@@ -7,12 +7,14 @@
 
 #include "archive.hpp"           // for Demangle
 #include "paje_trace.hpp"
+#include "ng_mpi.hpp"
 #include "profiler.hpp"
 #include "mpi_wrapper.hpp"
 
 extern const char *header;
 
 constexpr int MPI_PAJE_WRITER = 1;
+constexpr int ASSUMED_MPI_MAX_PROCESSOR_NAME = 256;
 
 namespace ngcore
 {
@@ -24,7 +26,7 @@ namespace ngcore
     if(id<NgProfiler::SIZE)
       return NgProfiler::GetName(id);
 
-    NgMPI_Comm comm(MPI_COMM_WORLD);
+    NgMPI_Comm comm(NG_MPI_COMM_WORLD);
     return NgProfiler::GetName(id-NgProfiler::SIZE*comm.Rank());
 #endif // PARALLEL
   }
@@ -39,6 +41,7 @@ namespace ngcore
   bool PajeTrace::trace_thread_counter = false;
   bool PajeTrace::trace_threads = true;
   bool PajeTrace::mem_tracing_enabled = true;
+  bool PajeTrace::write_paje_file = true;
 
   PajeTrace :: PajeTrace(int anthreads, std::string aname)
   {
@@ -70,9 +73,12 @@ namespace ngcore
 
     // sync start time when running in parallel
 #ifdef PARALLEL
-    NgMPI_Comm comm(MPI_COMM_WORLD);
-    for([[maybe_unused]] auto i : Range(5))
-        comm.Barrier();
+    if(MPI_Loaded())
+    {
+      NgMPI_Comm comm(NG_MPI_COMM_WORLD);
+      for([[maybe_unused]] auto i : Range(5))
+          comm.Barrier();
+    }
 #endif // PARALLEL
 
     start_time = GetTimeCounter();
@@ -112,11 +118,14 @@ namespace ngcore
     for(auto i : IntRange(n_memory_events_at_start, memory_events.size()))
       memory_events[i].time -= start_time;
 
-    NgMPI_Comm comm(MPI_COMM_WORLD);
-
+    NgMPI_Comm comm;
+  #ifdef PARALLEL
+    if(MPI_Loaded())
+      comm = NgMPI_Comm(NG_MPI_COMM_WORLD);
+  #endif
     if(comm.Size()==1)
     {
-      Write(tracefile_name);
+      Write();
     }
     else
     {
@@ -128,7 +137,7 @@ namespace ngcore
         event.timer_id += NgProfiler::SIZE*comm.Rank();
 
       if(comm.Rank() == MPI_PAJE_WRITER)
-        Write(tracefile_name);
+        Write();
       else
         SendData();
     }
@@ -219,9 +228,15 @@ namespace ngcore
           bool value_is_int = true;
 
           bool operator < (const PajeEvent & other) const {
+            /*
               // Same start and stop times can occur for very small tasks -> take "starting" events first (eg. PajePushState before PajePopState)
-              if(time == other.time)
-                return event_type < other.event_type;
+              if(time == other.time) {
+                if(value == other.value)  // same timer - first start, then stop
+                  return event_type < other.event_type;
+                else                              // different timers - first stop, then start
+                  return event_type > other.event_type;
+              }
+            */
               return (time < other.time);
           }
 
@@ -397,7 +412,7 @@ namespace ngcore
       void WriteEvents()
         {
           logger->info("Sorting traces...");
-          std::sort (events.begin(), events.end());
+          // std::stable_sort (events.begin(), events.end());
 
           logger->info("Writing traces... ");
           for (auto & event : events)
@@ -435,7 +450,16 @@ namespace ngcore
 
   NGCORE_API PajeTrace *trace;
 
-  void PajeTrace::Write( const std::string & filename )
+  void PajeTrace::Write( )
+    {
+      if(write_paje_file) WritePajeFile( tracefile_name );
+      WriteTimingChart();
+#ifdef NETGEN_TRACE_MEMORY
+      WriteMemoryChart("");
+#endif // NETGEN_TRACE_MEMORY
+    }
+
+  void PajeTrace::WritePajeFile( const std::string & filename )
     {
       auto n_events = jobs.size() + timer_events.size();
       for(auto & vtasks : tasks)
@@ -486,25 +510,25 @@ namespace ngcore
       std::vector <int> thread_aliases;
       std::vector<int> container_nodes;
 
-#ifdef PARALLEL
-      // Hostnames
-      NgMPI_Comm comm(MPI_COMM_WORLD);
-      // auto rank = comm.Rank();
-      auto nranks = comm.Size();
-      if(nranks>1)
+      NgMPI_Comm comm;
+  #ifdef PARALLEL
+      if(MPI_Loaded())
+        comm = NgMPI_Comm(NG_MPI_COMM_WORLD);
+      if(comm.Size()>1)
       {
-        nthreads = nranks;
+        auto comm = NgMPI_Comm(NG_MPI_COMM_WORLD);
+        nthreads = comm.Size();
         thread_aliases.reserve(nthreads);
 
-        std::array<char, MPI_MAX_PROCESSOR_NAME+1> ahostname;
+        std::array<char, ASSUMED_MPI_MAX_PROCESSOR_NAME+1> ahostname;
         int len;
-        MPI_Get_processor_name(ahostname.data(), &len);
+        NG_MPI_Get_processor_name(ahostname.data(), &len);
         std::string hostname = ahostname.data();
 
         std::map<std::string, int> host_map;
 
         std::string name;
-        for(auto i : IntRange(0, nranks))
+        for(auto i : IntRange(0, comm.Size()))
         {
           if(i!=MPI_PAJE_WRITER)
             comm.Recv(name, i, 0);
@@ -519,7 +543,7 @@ namespace ngcore
         }
       }
       else
-#endif // PARALLEL
+  #endif
       {
         container_nodes.reserve(num_nodes);
         for(int i=0; i<num_nodes; i++)
@@ -595,10 +619,9 @@ namespace ngcore
       for(auto id : timer_ids)
           timer_names[id] = GetTimerName(id);
 
-#ifdef PARALLEL
-      if(nranks>1)
+      if(comm.Size()>1)
       {
-        for(auto src : IntRange(0, nranks))
+        for(auto src : IntRange(0, comm.Size()))
         {
           if(src==MPI_PAJE_WRITER)
             continue;
@@ -617,7 +640,6 @@ namespace ngcore
           }
         }
       }
-#endif // PARALLEL
 
       for(auto id : timer_ids)
           timer_aliases[id] = paje.DefineEntityValue( state_type_timer, timer_names[id], -1 );
@@ -647,7 +669,7 @@ namespace ngcore
       for(auto & event : timer_events)
         {
           if(event.is_start)
-            paje.PushState( event.time, state_type_timer, timer_container_aliases[timerdepth++], timer_aliases[event.timer_id] );
+            paje.PushState( event.time, state_type_timer, timer_container_aliases[timerdepth++], timer_aliases[event.timer_id], event.custom_value );
           else
             paje.PopState( event.time, state_type_timer, timer_container_aliases[--timerdepth] );
         }
@@ -666,7 +688,7 @@ namespace ngcore
 
       if(user_events.size())
         {
-          std::sort (user_events.begin(), user_events.end());
+          // std::stable_sort (user_events.begin(), user_events.end());
 
           std::map<int, int> containers;
 
@@ -742,7 +764,7 @@ namespace ngcore
         }
 
 #ifdef PARALLEL
-      if(nranks>1)
+      if(comm.Size()>1)
       {
         for(auto & event : timer_events)
         {
@@ -758,7 +780,7 @@ namespace ngcore
         Array<bool> is_start;
         Array<int> thread_id;
 
-        for(auto src : IntRange(0, nranks))
+        for(auto src : IntRange(0, comm.Size()))
         {
           if(src==MPI_PAJE_WRITER)
             continue;
@@ -843,10 +865,6 @@ namespace ngcore
                 }
             }
         }
-      WriteTimingChart();
-#ifdef NETGEN_TRACE_MEMORY
-      WriteMemoryChart("");
-#endif // NETGEN_TRACE_MEMORY
       paje.WriteEvents();
     }
 
@@ -854,15 +872,15 @@ namespace ngcore
     {
 #ifdef PARALLEL
       // Hostname
-      NgMPI_Comm comm(MPI_COMM_WORLD);
+      NgMPI_Comm comm(NG_MPI_COMM_WORLD);
       // auto rank = comm.Rank();
       // auto nranks = comm.Size();
 
       std::string hostname;
         {
-          std::array<char, MPI_MAX_PROCESSOR_NAME+1> ahostname;
+          std::array<char, ASSUMED_MPI_MAX_PROCESSOR_NAME+1> ahostname;
           int len;
-          MPI_Get_processor_name(ahostname.data(), &len);
+          NG_MPI_Get_processor_name(ahostname.data(), &len);
           hostname = ahostname.data();
         }
 
@@ -955,10 +973,18 @@ namespace ngcore
     f.precision(4);
     f << R"CODE_(
 <head>
-  <script src="https://d3js.org/d3.v5.min.js"></script>
+  <script src="https://cdn.jsdelivr.net/npm/d3@7"></script>
   <script src="https://unpkg.com/sunburst-chart"></script>
 
-  <style>body { margin: 0 }</style>
+  <style>
+    body { margin: 0 }
+    .tooltip {
+      white-space: pre-line !important;
+      max-width: 800px !important;
+      word-wrap: break-word !important;
+      padding: 10px !important;
+    }
+  </style>
 )CODE_";
     if(!time_or_memory)
       f << "<title>Maximum Memory Consumption</title>\n";
@@ -1196,12 +1222,12 @@ namespace ngcore
           else
               id = jobs_map[name];
 
-          events.push_back(TimerEvent{-1, job.start_time, true, id});
-          events.push_back(TimerEvent{-1, job.stop_time, false, id});
+          events.push_back(TimerEvent{job.start_time, -1, id, -1, true});
+          events.push_back(TimerEvent{job.stop_time, -1, id, -1, false});
           stop_time = std::max(job.stop_time, stop_time);
       }
 
-      std::sort (events.begin(), events.end());
+      std::stable_sort (events.begin(), events.end());
 
       root.size = 1000.0*static_cast<double>(stop_time) * seconds_per_tick;
       root.calls = 1;
