@@ -71,9 +71,9 @@ void Ng_LoadGeometry (const char * filename)
       return;
     }
 
-  for (int i = 0; i < geometryregister.Size(); i++)
+  for (auto loader : GeometryRegister())
     {
-      NetgenGeometry * hgeom = geometryregister[i]->Load (filename);
+      NetgenGeometry * hgeom = loader->Load (filename);
       if (hgeom)
 	{
           ng_geometry.reset (hgeom);
@@ -94,7 +94,7 @@ void Ng_LoadMeshFromStream ( istream & input )
   mesh -> Load(input);
 
   SetGlobalMesh (mesh);
-  ng_geometry = geometryregister.LoadFromMeshFile (input);
+  ng_geometry = GeometryRegister().LoadFromMeshFile (input);
 
   if (!ng_geometry)
     ng_geometry = make_shared<NetgenGeometry>();
@@ -249,7 +249,7 @@ void Ng_LoadMesh (const char * filename, ngcore::NgMPI_Comm comm)
   shared_ptr<NetgenGeometry> geo;
   if(buf.Size()) { // if we had geom-info in the file, take it
     istringstream geom_infile(string((const char*)&buf[0], buf.Size()));
-    geo = geometryregister.LoadFromMeshFile(geom_infile);
+    geo = GeometryRegister().LoadFromMeshFile(geom_infile);
   }
   if(geo!=nullptr) {
     ng_geometry = geo;
@@ -651,14 +651,14 @@ int Ng_FindElementOfPoint (double * p, double * lami, int build_searchtree,
     {
       Point3d p3d(p[0], p[1], p[2]);
       ind = 
-	mesh->GetElementOfPoint(p3d, lami, dummy, build_searchtree != 0);
+	mesh->GetElementOfPoint(p3d, lami, dummy, build_searchtree != 0) + 1;
     }
   else
     {
       double lam3[3];
       Point3d p2d(p[0], p[1], 0);
       ind = 
-	mesh->GetElementOfPoint(p2d, lam3, dummy, build_searchtree != 0);
+	mesh->GetSurfaceElementOfPoint(p2d, lam3, dummy, build_searchtree != 0) + 1;
 
       if (ind > 0)
 	{
@@ -697,7 +697,7 @@ int Ng_FindSurfaceElementOfPoint (double * p, double * lami, int build_searchtre
     {
       Point3d p3d(p[0], p[1], p[2]);
       ind = 
-	mesh->GetSurfaceElementOfPoint(p3d, lami, dummy, build_searchtree != 0);
+	mesh->GetSurfaceElementOfPoint(p3d, lami, dummy, build_searchtree != 0) + 1;
     }
   else
     {
@@ -1887,9 +1887,9 @@ void Ng_GetPeriodicEdges (int idnr, int * pairs)
 
 
 
-void Ng_PushStatus (const char * str)
+void Ng_PushStatus (const std::string& str)
 {
-  PushStatus (MyStr (str));
+  PushStatus (str);
 }
 
 void Ng_PopStatus ()
@@ -1902,12 +1902,15 @@ void Ng_SetThreadPercentage (double percent)
   SetThreadPercent (percent);
 }
 
-void Ng_GetStatus (char ** str, double & percent)
+void Ng_GetStatus (std::string & str, double & percent)
 {
+  /*
   MyStr s;
   GetStatus(s,percent);
   *str = new char[s.Length()+1];
-  strcpy(*str,s.c_str());  
+  strcpy(*str,s.c_str());
+  */
+  GetStatus (str, percent);
 }
 
 
