@@ -725,34 +725,32 @@ namespace netgen
   
   int Ngx_Mesh :: GetParentElement (int ei) const
   {
-      ei++;
-      if (mesh->GetDimension() == 3)
+    if (mesh->GetDimension() == 3)
       {
-          if (ei <= mesh->mlparentelement.Size())
-              return mesh->mlparentelement.Get(ei)-1;
+        if (ei < mesh->mlparentelement.Size())
+          return mesh->mlparentelement[ei];
       }
-      else
+    else
       {
-          if (ei <= mesh->mlparentsurfaceelement.Size())
-              return mesh->mlparentsurfaceelement.Get(ei)-1;
+        if (ei < mesh->mlparentsurfaceelement.Size())
+          return mesh->mlparentsurfaceelement[ei];
       }
-      return -1;
+    return -1;
   }
 
 
   int Ngx_Mesh :: GetParentSElement (int ei) const
   {
-      ei++;
-      if (mesh->GetDimension() == 3)
+    if (mesh->GetDimension() == 3)
       {
-          if (ei <= mesh->mlparentsurfaceelement.Size())
-              return mesh->mlparentsurfaceelement.Get(ei)-1;
+        if (ei < mesh->mlparentsurfaceelement.Size())
+          return mesh->mlparentsurfaceelement[ei];
       }
-      else
+    else
       {
-          return -1;
+        return -1;
       }
-      return -1;
+    return -1;
   }
 
   int Ngx_Mesh :: GetNIdentifications () const
@@ -1010,71 +1008,31 @@ namespace netgen
   DLL_HEADER int Ngx_Mesh :: FindElementOfPoint <1> 
   (double * hp, double * lami,
    bool build_searchtree, 
-   int * const indices, int numind) const
+   int * const indices, int numind, double tol) const
 
   {
-    switch (mesh->GetDimension())
+    Point<3> p(hp[0], 0., 0.);
+    if(mesh->GetDimension() > 1)
+      p[1] = hp[1];
+    if(mesh->GetDimension() == 3)
+      p[2] = hp[2];
+
+    for (SegmentIndex si = 0; si < mesh->GetNSeg(); si++)
       {
-      case 1:
-        {
-          Point<3> p(hp[0], 0,0);
-          for (SegmentIndex si = 0; si < mesh->GetNSeg(); si++)
-            {
-              auto & seg = (*mesh)[si];
-              Point<3> p1 = (*mesh)[seg[0]];
-              Point<3> p2 = (*mesh)[seg[1]];
-              double lam = (p(0)-p1(0)) / (p2(0)-p1(0));
-              if (lam >= -1e-10 && lam <= 1+1e-10)
-                {
-                  lami[0] = 1-lam;
-                  return si;
-                }
-            }
-        }
-        break;
-      case 2:
-        {
-          Point<3> p(hp[0], hp[1],0);
-          try
-            {
-              auto ind = mesh->GetSurfaceElementOfPoint(p, lami, nullptr,
-                                                        build_searchtree);
-              return ind - 1;
-            }
-          catch(NgException e) // quads not implemented curved yet
-            {
-          for (SegmentIndex si = 0; si < mesh->GetNSeg(); si++)
-            {
-              auto & seg = (*mesh)[si];
-              Point<3> p1 = (*mesh)[seg[0]];
-              Point<3> p2 = (*mesh)[seg[1]];
-              double lam;
-              double r;
-              if (fabs(p2[0]-p1[0]) >= fabs(p2[1]-p1[1]))
-                {
-                  lam = (p[0]-p1[0])/(p2[0]-p1[0]);
-                  r = p[1] - p1[1] - lam*(p2[1]-p1[1]);
-                }
-              else
-                {
-                  lam = (p[1]-p1[1])/(p2[1]-p1[1]);
-                  r = p[0] - p1[0] - lam*(p2[0]-p1[0]);
-                }
-              if ( lam >= -1e-10 && lam <= 1+1e-10 && fabs(r) <= 1e-10 )
-                {
-                  lami[0] = 1-lam;
-                  return si;
-                }
-            }
-            }
-        }
-        break;
-      case 3:
-      default:
-        throw Exception("FindElementOfPoint<1> only implemented for mesh-dimension 1 and 2!");
-        break;
+        auto & seg = (*mesh)[si];
+        Point<3> p1 = (*mesh)[seg[0]];
+        Point<3> p2 = (*mesh)[seg[1]];
+        Vec<3> v1 = p2-p1;
+        Vec<3> v2 = p-p1;
+        double lam = v1*v2 / v1.Length2();
+        double lam2 = (v2 - lam * v1).Length() / v1.Length();
+
+        if (lam >= -1e-10 && lam <= 1+1e-10 && lam2 < 1e-10)
+          {
+            lami[0] = 1-lam;
+            return si;
+          }
       }
- 
     return -1;
   }
 
@@ -1082,40 +1040,29 @@ namespace netgen
   DLL_HEADER int Ngx_Mesh :: FindElementOfPoint <2> 
   (double * p, double * lami,
    bool build_searchtree, 
-   int * const indices, int numind) const
+   int * const indices, int numind, double tol) const
 
   {
-    NgArray<int> dummy(numind);
-    for (int i = 0; i < numind; i++) dummy[i] = indices[i]+1;
-    
+    Point<3> pp(p[0], p[1], 0.);
+    if(mesh->GetDimension() == 3)
+      pp[2] = p[2];
+    FlatArray<int> ind(numind, indices);
     double lam3[3];
-    int ind;
-
-    if (mesh->GetDimension() == 2)
+    auto elnr = mesh->GetSurfaceElementOfPoint(pp, lam3, ind, build_searchtree);
+    if(elnr.IsValid())
       {
-        Point<3> p2d(p[0], p[1], 0);
-        ind = mesh->GetElementOfPoint(p2d, lam3, &dummy, build_searchtree);
-      }
-    else
-      {
-        Point3d p3d(p[0], p[1], p[2]);
-        ind = mesh->GetSurfaceElementOfPoint(p3d, lam3, &dummy, build_searchtree);
-      }
-    
-    if (ind > 0)
-      {
-        if(mesh->SurfaceElement(ind).GetType()==QUAD || mesh->SurfaceElement(ind).GetType()==TRIG6)
+        if((*mesh)[elnr].GetType() == QUAD || (*mesh)[elnr].GetType() == TRIG6)
           {
             lami[0] = lam3[0];
             lami[1] = lam3[1];
           }
-        else 
+        else
           {
             lami[0] = 1-lam3[0]-lam3[1];
             lami[1] = lam3[0];
           }
       }
-    return ind-1;
+    return elnr;
   }
 
 
@@ -1123,16 +1070,13 @@ namespace netgen
   DLL_HEADER int Ngx_Mesh :: FindElementOfPoint <3>
   (double * p, double * lami,
    bool build_searchtree, 
-   int * const indices, int numind) const
+   int * const indices, int numind,
+   double tol) const
 
   {
-    NgArray<int> dummy(numind);
-    for (int i = 0; i < numind; i++) dummy[i] = indices[i]+1;
-    
-    Point<3> p3d(p[0], p[1], p[2]);
-    int ind = 
-      mesh->GetElementOfPoint(p3d, lami, &dummy, build_searchtree);
-    return ind-1;
+    Point<3> pp(p[0], p[1], p[2]);
+    FlatArray<int> ind(numind, indices);
+    return mesh->GetElementOfPoint(pp, lami, ind, build_searchtree, true, tol);
   }
 
   void Ngx_Mesh :: Curve (int order)
@@ -1234,7 +1178,7 @@ namespace netgen
   {
     NgLock meshlock (mesh->MajorMutex(), true);
     Refinement & ref = const_cast<Refinement&> (mesh->GetGeometry()->GetRefinement());
-    ::netgen::HPRefinement (*mesh, &ref, SPLIT_ALFELD, 1, 1.0/3.0, true, true);
+    ::netgen::HPRefinement (*mesh, &ref, SPLIT_ALFELD, 1, 1.0 / (mesh->GetDimension()+1), true, true);
   }
 
   
@@ -1249,9 +1193,15 @@ int Ngx_Mesh::GetElementOrder (int enr) const
 void Ngx_Mesh::GetElementOrders (int enr, int * ox, int * oy, int * oz) const
 {
   if (mesh->GetDimension() == 3)
-    mesh->VolumeElement(enr).GetOrder(*ox, *oy, *oz);
+    {
+      ElementIndex ei = IndexBASE<ElementIndex>() + enr-1;
+      mesh->VolumeElement(ei).GetOrder(*ox, *oy, *oz);
+    }
   else
-    mesh->SurfaceElement(enr).GetOrder(*ox, *oy, *oz);
+    {
+      SurfaceElementIndex sei = IndexBASE<SurfaceElementIndex>() + enr-1;      
+      mesh->SurfaceElement(sei).GetOrder(*ox, *oy, *oz);
+    }
 }
 
 void Ngx_Mesh::SetElementOrder (int enr, int order)
@@ -1297,6 +1247,36 @@ int Ngx_Mesh::GetClusterRepElement (int pi) const
 }
 
 
+int Ngx_Mesh::GetElement_Faces (int elnr, int * faces, int * orient) const
+{
+  const MeshTopology & topology = mesh->GetTopology();
+  if (mesh->GetDimension() == 3)
+    {
+      int num = topology.GetElementFaces (elnr+1, faces, orient);
+      for (int i = 0; i < num; i++)
+        faces[i]--;
+      return num;
+    }
+  else
+    {
+      faces[0] = elnr;
+      if (orient) orient[0] = 0;
+      return 1;
+    }
+}
+
+int Ngx_Mesh::GetSurfaceElement_Face (int selnr, int * orient) const
+{
+  if (mesh->GetDimension() == 3)
+    {
+      const MeshTopology & topology = mesh->GetTopology();
+      if (orient)
+	*orient = topology.GetSurfaceElementFaceOrientation (selnr+1);
+      return topology.GetFace (SurfaceElementIndex(selnr));
+    }
+  return -1;
+}
+
 
 
 //HERBERT: falsche Anzahl von Argumenten
