@@ -30,8 +30,18 @@ namespace netgen
       }
   };
 
-  DLL_HEADER GeometryRegisterArray geometryregister;
-  //DLL_HEADER NgArray<GeometryRegister*> geometryregister;
+  void ShapeProperties :: DoArchive( Archive & ar )
+  {
+      ar & name & col & maxh & hpref & layer;
+      if(ar.GetVersion("netgen") > "v6.2.2506-14")
+          ar & partition & quad_dominated;
+  }
+
+  GeometryRegisterArray& GeometryRegister()
+  {
+    static GeometryRegisterArray geometryregister;
+    return geometryregister;
+  }
 
   GeometryRegister :: ~GeometryRegister()
   { ; }
@@ -474,7 +484,7 @@ namespace netgen
       }
 
     for(const auto& mspnt : mparam.meshsize_points)
-      mesh.RestrictLocalH(mspnt.pnt, mspnt.h);
+      mesh.RestrictLocalH(mspnt.pnt, mspnt.h, mspnt.layer);
 
     mesh.LoadLocalMeshSize(mparam.meshsizefilename);
   }
@@ -1220,6 +1230,7 @@ namespace netgen
       {
         PrintMessage(3, "Optimization step ", i);
         meshopt.SetFaceIndex(k+1);
+        meshopt.SetMetricWeight (mparam.elsizeweight);
         int innerstep = 0;
         for(auto optstep : mparam.optimize2d)
           {
@@ -1314,6 +1325,13 @@ namespace netgen
     if(multithread.terminate || mparam.perfstepsend <= MESHCONST_MESHEDGES)
       return 0;
 
+    if(dimension == 1)
+      {
+        FinalizeMesh(*mesh);
+        mesh->SetDimension(1);
+        return 0;
+      }
+
     if (mparam.perfstepsstart <= MESHCONST_MESHSURFACE)
       {
         MeshSurface(*mesh, mparam);
