@@ -14,7 +14,7 @@ namespace netgen
 
 struct SpecialPointException : public Exception
 {
-  SpecialPointException()
+  SpecialPointException ()
     : Exception("") {}
 };
 
@@ -101,14 +101,14 @@ Vec<3> CalcGrowthVector (FlatArray<Vec<3>> ns)
   return gw;
 }
 
-SpecialBoundaryPoint ::GrowthGroup ::GrowthGroup(FlatArray<int> faces_,
-                                                 FlatArray<Vec<3>> normals)
+SpecialBoundaryPoint ::GrowthGroup ::GrowthGroup (FlatArray<int> faces_,
+                                                  FlatArray<Vec<3>> normals)
 {
   faces = faces_;
   growth_vector = CalcGrowthVector(normals);
 }
 
-SpecialBoundaryPoint ::SpecialBoundaryPoint(
+SpecialBoundaryPoint ::SpecialBoundaryPoint (
   const std::map<int, Vec<3>>& normals)
 {
   // find opposing face normals
@@ -146,7 +146,7 @@ SpecialBoundaryPoint ::SpecialBoundaryPoint(
   growth_groups.Append(GrowthGroup(g2_faces, normals2));
 }
 
-Vec<3> BoundaryLayerTool ::getEdgeTangent(PointIndex pi, int edgenr, FlatArray<Segment*> segs)
+Vec<3> BoundaryLayerTool ::getEdgeTangent (PointIndex pi, int edgenr, FlatArray<Segment*> segs)
 {
   Vec<3> tangent = 0.0;
   ArrayMem<PointIndex, 2> pts;
@@ -171,7 +171,7 @@ Vec<3> BoundaryLayerTool ::getEdgeTangent(PointIndex pi, int edgenr, FlatArray<S
   return tangent.Normalize();
 }
 
-void BoundaryLayerTool ::LimitGrowthVectorLengths()
+void BoundaryLayerTool ::LimitGrowthVectorLengths ()
 {
   static Timer tall("BoundaryLayerTool::LimitGrowthVectorLengths");
   RegionTimer rtall(tall);
@@ -220,7 +220,7 @@ bool HaveSingleSegments (const Mesh& mesh)
 
 // duplicates segments (and sets seg.si accordingly) to have a unified data
 // structure for all geometry types
-void BuildSegments (Mesh& mesh, bool have_single_segments, Array<Segment> & segments, Array<Segment> & free_segments)
+void BuildSegments (Mesh& mesh, bool have_single_segments, Array<Segment>& segments, Array<Segment>& free_segments)
 {
   // auto& topo = mesh.GetTopology();
 
@@ -234,11 +234,11 @@ void BuildSegments (Mesh& mesh, bool have_single_segments, Array<Segment> & segm
           free_segments.Append(seg);
           continue;
         }
-      if(!have_single_segments)
-      {
-        segments.Append(seg);
-        continue;
-      }
+      if (!have_single_segments)
+        {
+          segments.Append(seg);
+          continue;
+        }
       mesh.GetTopology().GetSegmentSurfaceElements(segi + 1, surf_els);
       for (auto seli : surf_els)
         {
@@ -284,8 +284,8 @@ void MergeAndAddSegments (Mesh& mesh, FlatArray<Segment> segments, FlatArray<Seg
     addSegment(seg);
 }
 
-BoundaryLayerTool::BoundaryLayerTool(Mesh& mesh_,
-                                     const BoundaryLayerParameters& params_)
+BoundaryLayerTool::BoundaryLayerTool (Mesh& mesh_,
+                                      const BoundaryLayerParameters& params_)
   : mesh(mesh_), topo(mesh_.GetTopology()), params(params_)
 {
   static Timer timer("BoundaryLayerTool::ctor");
@@ -318,7 +318,7 @@ BoundaryLayerTool::BoundaryLayerTool(Mesh& mesh_,
     si_map[i] = i;
 }
 
-void BoundaryLayerTool ::CreateNewFaceDescriptors()
+void BoundaryLayerTool ::CreateNewFaceDescriptors ()
 {
   surfacefacs.SetSize(nfd_old + 1);
   surfacefacs = 0.0;
@@ -360,7 +360,7 @@ void BoundaryLayerTool ::CreateNewFaceDescriptors()
       throw Exception("Surface " + to_string(si) + " is not a boundary of the domain to be grown into!");
 }
 
-void BoundaryLayerTool ::CreateFaceDescriptorsSides()
+void BoundaryLayerTool ::CreateFaceDescriptorsSides ()
 {
   if (insert_only_volume_elements)
     return;
@@ -400,7 +400,7 @@ void BoundaryLayerTool ::CreateFaceDescriptorsSides()
     }
 }
 
-void BoundaryLayerTool ::CalculateGrowthVectors()
+void BoundaryLayerTool ::CalculateGrowthVectors ()
 {
   growthvectors.SetSize(np);
   growthvectors = 0.;
@@ -457,7 +457,7 @@ void BoundaryLayerTool ::CalculateGrowthVectors()
 }
 
 Array<Array<pair<SegmentIndex, int>>, SegmentIndex>
-BoundaryLayerTool ::BuildSegMap()
+BoundaryLayerTool ::BuildSegMap ()
 {
   // Bit array to keep track of segments already processed
   BitArray segs_done(nseg + 1);
@@ -536,7 +536,7 @@ BoundaryLayerTool ::BuildSegMap()
   return segmap;
 }
 
-BitArray BoundaryLayerTool ::ProjectGrowthVectorsOnSurface()
+BitArray BoundaryLayerTool ::ProjectGrowthVectorsOnSurface ()
 {
   BitArray in_surface_direction(nfd_old + 1);
   in_surface_direction.Clear();
@@ -596,7 +596,7 @@ BitArray BoundaryLayerTool ::ProjectGrowthVectorsOnSurface()
   return in_surface_direction;
 }
 
-void BoundaryLayerTool ::InsertNewElements(
+void BoundaryLayerTool ::InsertNewElements (
   FlatArray<Array<pair<SegmentIndex, int>>, SegmentIndex> segmap,
   const BitArray& in_surface_direction)
 {
@@ -879,61 +879,61 @@ void BoundaryLayerTool ::InsertNewElements(
   auto p2el = mesh.CreatePoint2ElementTable();
   for (SurfaceElementIndex si = 0; si < nse; si++)
     {
-      // copy because surfaceels array will be resized!
       const auto sel = mesh[si];
-      if (moved_surfaces.Test(sel.GetIndex()))
+      const auto iface = sel.GetIndex();
+
+      if (moved_surfaces.Test(iface))
         {
-          Array<PointIndex> points(sel.PNums());
-          if (surfacefacs[sel.GetIndex()] > 0)
+          const auto np = sel.GetNP();
+          ArrayMem<PointIndex, 4> points(sel.PNums());
+          if (surfacefacs[iface] > 0)
             Swap(points[0], points[2]);
           ArrayMem<int, 4> groups(points.Size());
           for (auto i : Range(points))
-            groups[i] = getClosestGroup(sel[i], si);
+            groups[i] = getClosestGroup(points[i], si);
           bool add_volume_element = true;
-          for (auto pi : sel.PNums())
+          for (auto pi : points)
             if (numGroups(pi) > 1)
               add_volume_element = false;
+
+          Element el(2 * np);
+          el.PNums().Range(np, 2 * np) = points;
+          auto new_index = new_mat_nrs[iface];
+          if (new_index == -1)
+            throw Exception("Boundary " + ToString(iface) + " with name " + mesh.GetBCName(iface - 1) + " extruded, but no new material specified for it!");
+          el.SetIndex(new_index);
+
           for (auto j : Range(par_heights))
             {
-              auto eltype = points.Size() == 3 ? PRISM : HEX;
-              Element el(eltype);
-              for (auto i : Range(points))
-                el[i] = points[i];
-              for (auto i : Range(points))
-                points[i] = newPoint(sel.PNums()[i], j, groups[i]);
-              if (surfacefacs[sel.GetIndex()] > 0)
-                Swap(points[0], points[2]);
-              for (auto i : Range(points))
-                el[sel.PNums().Size() + i] = points[i];
-              auto new_index = new_mat_nrs[sel.GetIndex()];
-              if (new_index == -1)
-                throw Exception("Boundary " + ToString(sel.GetIndex()) + " with name " + mesh.GetBCName(sel.GetIndex() - 1) + " extruded, but no new material specified for it!");
-              el.SetIndex(new_mat_nrs[sel.GetIndex()]);
+              el.PNums().Range(0, np) = el.PNums().Range(np, 2 * np);
+              for (auto i : Range(np))
+                el[np + i] = newPoint(points[i], j, groups[i]);
               if (add_volume_element)
                 mesh.AddVolumeElement(el);
               else
                 {
                   // Let the volume mesher fill the hole with pyramids/tets
-                  // To insert pyramids, we need close surface identifications on open
-                  // quads
-                  for (auto i : Range(points))
-                    if (numGroups(sel[i]) == 1)
+                  // To insert pyramids, we need close surface identifications on open quads
+                  for (auto i : Range(np))
+                    if (numGroups(el[i]) == 1)
                       {
                         auto pi0 = el[i];
-                        auto pi1 = el[i + points.Size()];
+                        auto pi1 = el[np + i];
                         auto nr = identifications.Get(pi0, pi1);
                         if (nr == 0)
-                          identifications.Add(el[i], el[i + points.Size()], identnr);
+                          identifications.Add(pi0, pi1, identnr);
                       }
                 }
             }
           Element2d newel = sel;
-          for (auto i : Range(points))
-            newel[i] = newPoint(sel[i], -1, groups[i]);
-          newel.SetIndex(si_map[sel.GetIndex()]);
+          for (auto i : Range(np))
+            newel[i] = newPoint(points[i], -1, groups[i]);
+          if (surfacefacs[iface] > 0)
+            Swap(newel[0], newel[2]); // swap back
+          newel.SetIndex(si_map[iface]);
           new_sels.Append(newel);
         }
-      if (is_boundary_moved.Test(sel.GetIndex()))
+      if (is_boundary_moved.Test(iface))
         {
           auto& sel = mesh[si];
           for (auto& p : sel.PNums())
@@ -1065,7 +1065,7 @@ void BoundaryLayerTool ::InsertNewElements(
     }
 }
 
-void BoundaryLayerTool ::SetDomInOut()
+void BoundaryLayerTool ::SetDomInOut ()
 {
   if (insert_only_volume_elements)
     return;
@@ -1081,7 +1081,7 @@ void BoundaryLayerTool ::SetDomInOut()
       }
 }
 
-void BoundaryLayerTool ::SetDomInOutSides()
+void BoundaryLayerTool ::SetDomInOutSides ()
 {
   // Set the domin/domout entries for face descriptors on the "side" of new boundary layers
   if (insert_only_volume_elements)
@@ -1155,8 +1155,24 @@ void BoundaryLayerTool ::SetDomInOutSides()
     }
 }
 
-void BoundaryLayerTool ::AddSegments()
+void BoundaryLayerTool ::AddSegments ()
 {
+  if (insert_only_volume_elements)
+    {
+      if (params.disable_curving)
+        {
+          auto is_mapped = [&] (PointIndex pi) {
+            return pi >= mapto.Range().Next() || mapto[pi].Size() > 0;
+          };
+          for (auto& seg : old_segments)
+            if (is_mapped(seg[0]) || is_mapped(seg[1]))
+              {
+                seg.epgeominfo[0].edgenr = -1;
+                seg.epgeominfo[1].edgenr = -1;
+              }
+        }
+    }
+
   auto& new_segs =
     insert_only_volume_elements ? new_segments_on_moved_bnd : new_segments;
 
@@ -1199,14 +1215,14 @@ void BoundaryLayerTool ::AddSegments()
     mesh.AddSegment(seg);
 }
 
-void BoundaryLayerTool ::AddSurfaceElements()
+void BoundaryLayerTool ::AddSurfaceElements ()
 {
   for (auto& sel :
        insert_only_volume_elements ? new_sels_on_moved_bnd : new_sels)
     mesh.AddSurfaceElement(sel);
 }
 
-void BoundaryLayerTool ::ProcessParameters()
+void BoundaryLayerTool ::ProcessParameters ()
 {
   if (int* bc = get_if<int>(&params.boundary); bc)
     {
@@ -1374,7 +1390,7 @@ void BoundaryLayerTool ::ProcessParameters()
     domains.Invert();
 }
 
-void BoundaryLayerTool ::Perform()
+void BoundaryLayerTool ::Perform ()
 {
   if (domains.NumSet() == 0)
     return;
@@ -1425,6 +1441,11 @@ void BoundaryLayerTool ::Perform()
         }
     }
 
+  // there is still a bug with segment edge numbers in moved boundaries.
+  // As a workaround, don't add them at all if only volume elements are inserted
+  if (insert_only_volume_elements)
+    mesh.LineSegments() = old_segments;
+
   mesh.CalcSurfacesOfNode();
   mesh.GetTopology().ClearEdges();
   mesh.SetNextMajorTimeStamp();
