@@ -1,4 +1,3 @@
-#include "pybind11/pytypes.h"
 #ifdef NG_PYTHON
 
 #include <regex>
@@ -9,6 +8,7 @@
 
 #include <mystdlib.h>
 #include "meshing.hpp"
+#include "boundarylayer.hpp"
 // #include <csg.hpp>
 // #include <geometry2d.hpp>
 #include <../interface/rw_medit.hpp>
@@ -67,11 +67,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
 
   m.def("_GetStatus", []()
         {
-          MyStr s; double percent;
+          std::string s; double percent;
           GetStatus(s, percent);
           return py::make_tuple(s.c_str(), percent);
         });
-  m.def("_PushStatus", [](string s) { PushStatus(MyStr(s)); });
+  m.def("_PushStatus", [](string s) { PushStatus(s); });
   m.def("_SetThreadPercentage", [](double percent) { SetThreadPercent(percent); });
 
   py::enum_<Identifications::ID_TYPE>(m,"IdentificationType")
@@ -180,6 +180,34 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
 
   py::implicitly_convertible<py::tuple, Vec<3>>();
 
+  py::class_<Mat<3,3>>(m, "Mat33")
+    .def(py::init([](py::tuple m)
+    {
+      if(m.size() != 9)
+        throw std::length_error("Invalid dimension of input array!");
+      Mat<3,3> mat;
+      for(int i = 0; i < 3; i++)
+        for(int j = 0; j < 3; j++)
+          mat(i,j) = m[i*3+j].cast<double>();
+      return mat;
+    }))
+    .def("__getitem__", [](Mat<3,3>& mat, py::tuple index)
+    {
+      if(index.size() != 2)
+        throw std::length_error("Invalid dimension of input array!");
+      return mat(index[0].cast<int>(), index[1].cast<int>());
+    })
+    .def("__setitem__", [](Mat<3,3>& mat, py::tuple index, double val)
+    {
+      if(index.size() != 2)
+        throw std::length_error("Invalid dimension of input array!");
+      mat(index[0].cast<int>(), index[1].cast<int>()) = val;
+    })
+    .def("__str__", &ToString<Mat<3,3>>)
+    ;
+
+  py::implicitly_convertible<py::tuple, Mat<3,3>>();
+
   m.def ("Vec", FunctionPointer
            ([] (double x, double y, double z) { return global_trafo(Vec<3>(x,y,z)); }));
   m.def("Vec", [](py::array_t<double> np_array)
@@ -204,6 +232,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
     .def("__mul__", [](Transformation<3> a, Transformation<3> b)->Transformation<3>
          { Transformation<3> res; res.Combine(a,b); return res; })
     .def("__call__", [] (Transformation<3> trafo, Point<3> p) { return trafo(p); })
+    .def_property("mat", &Transformation<3>::GetMatrix,
+                  [](Transformation<3>& self, const Mat<3,3>& mat)
+                  {
+                    self.GetMatrix() = mat;
+                  })
     ;
 
   m.def ("GetTransformation", [] () { return global_trafo; });
@@ -490,10 +523,8 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                     for (int i = 0; i < 2; i++)
                       (*newel)[i] = py::extract<PointIndex>(vertices[i])();
                     newel -> si = index;
-                    newel -> edgenr = edgenr;
                     newel -> epgeominfo[0].edgenr = edgenr;
                     newel -> epgeominfo[1].edgenr = edgenr;
-                    // needed for codim2 in 3d
                     newel -> edgenr = index;
                     for(auto i : Range(len(trignums)))
                       newel->geominfo[i].trignum = py::cast<int>(trignums[i]);
@@ -536,14 +567,24 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                                      li.append (py::cast(self.surfnr2));
                                      return li;
                                    }))
-    .def_property_readonly("index", FunctionPointer([](const Segment &self) -> size_t
-		  {
-		    return self.si;
-		  }))
-    .def_property_readonly("edgenr", FunctionPointer([](const Segment & self) -> size_t
-						     {
-						       return self.edgenr;
-						     }))
+    .def_property("index",
+                  [](const Segment &self) -> size_t
+                  {
+                    return self.si;
+                  },
+                  [](Segment& self, int index)
+                  {
+                    self.si = index;
+                  })
+    .def_property("edgenr",
+                  [](const Segment & self) -> size_t
+                  {
+                    return self.edgenr;
+                  },
+                  [](Segment& self, int edgenr)
+                  {
+                    self.edgenr = edgenr;
+                  })
     .def_property("singular",
                   [](const Segment & seg) { return seg.singedge_left; },
                   [](Segment & seg, double sing) { seg.singedge_left = sing; seg.singedge_right=sing; })
@@ -558,7 +599,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
           py::format_descriptor<int[3]>::format(),
           py::detail::npy_format_descriptor<int[3]>::dtype() },
         py::detail::field_descriptor {
-          "index", offsetof(Segment, edgenr), sizeof(int),
+          "index", offsetof(Segment, si), sizeof(int),
+          py::format_descriptor<int>::format(),
+          py::detail::npy_format_descriptor<int>::dtype() },
+        py::detail::field_descriptor {
+          "edgenr", offsetof(Segment, edgenr), sizeof(int),
           py::format_descriptor<int>::format(),
           py::detail::npy_format_descriptor<int>::dtype() },
       });
@@ -623,7 +668,7 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                   [](const FaceDescriptor& self)
                   {
                     auto sc = self.SurfColour();
-                    return py::make_tuple(sc[0], sc[1], sc[2]);
+                    return py::make_tuple(sc[0], sc[1], sc[2], sc[3]);
                   },
                   [](FaceDescriptor& self, py::tuple col)
                   {
@@ -649,8 +694,14 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                   })
     ;
 
+  py::implicitly_convertible< int, SurfaceElementIndex>();
+  PYBIND11_NUMPY_DTYPE(SurfaceElementIndex, i);
+  ExportArray<SurfaceElementIndex, SurfaceElementIndex>(m);
+  
+  py::implicitly_convertible< int, ElementIndex>();
+  PYBIND11_NUMPY_DTYPE(ElementIndex, i);
+  ExportArray<ElementIndex, ElementIndex>(m);
   
-
   ExportArray<Element,ElementIndex>(m);
   ExportArray<Element2d,SurfaceElementIndex>(m);
   ExportArray<Segment,SegmentIndex>(m);
@@ -870,7 +921,7 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
 	    shared_ptr<NetgenGeometry> geo;
 	    if(buf.Size()) { // if we had geom-info in the file, take it
 	      istringstream geom_infile(string((const char*)buf.Data(), buf.Size()));
-	      geo = geometryregister.LoadFromMeshFile(geom_infile);
+	      geo = GeometryRegister().LoadFromMeshFile(geom_infile);
 	    }
 	    if(geo!=nullptr) mesh->SetGeometry(geo);
 	    else if(ng_geometry!=nullptr) mesh->SetGeometry(ng_geometry);
@@ -920,11 +971,13 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            );
       })
     .def_property_readonly("parentelements", [](Mesh & self) {
-      return FlatArray<int>(self.mlparentelement.Size(), &self.mlparentelement[0]);
+      // return FlatArray<int>(self.mlparentelement.Size(), &self.mlparentelement[0]);
+      return FlatArray(self.mlparentelement);
     }, py::keep_alive<0,1>())
     .def_property_readonly("parentsurfaceelements", [](Mesh & self) {
-      return FlatArray<int>(self.mlparentsurfaceelement.Size(),
-                            &self.mlparentsurfaceelement[0]);
+      // return FlatArray<int>(self.mlparentsurfaceelement.Size(),
+      // &self.mlparentsurfaceelement[0]);
+      return FlatArray(self.mlparentsurfaceelement);      
     }, py::keep_alive<0,1>())
     .def_property_readonly("macromesh", [](Mesh & self) {
       auto coarsemesh = make_shared<Mesh>();
@@ -947,7 +1000,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
     .def("FaceDescriptor", static_cast<FaceDescriptor&(Mesh::*)(int)> (&Mesh::GetFaceDescriptor),
          py::return_value_policy::reference)
     .def("GetNFaceDescriptors", &Mesh::GetNFD)
-    
+    .def("RestrictLocalH", [](Mesh& self, const Point<3>& pnt, double maxh,
+                              int layer)
+    {
+      self.RestrictLocalH(pnt, maxh, layer);
+    }, py::arg("p"), py::arg("h"), py::arg("layer")=1)
     .def("FaceDescriptors", 
          // static_cast<Array<Element>&(Mesh::*)()> (&Mesh::FaceDescriptors),
          &Mesh::FaceDescriptors,         
@@ -988,10 +1045,22 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
             return self.AddSurfaceElement (el);
           })
 
-    .def ("Add", [](Mesh & self, const Segment & el)
+    .def ("Add", [](Mesh & self, const Segment & el, bool project_geominfo)
           {
+            if (project_geominfo)
+              {
+                auto &p1 = self[el[0]];
+                auto &p2 = self[el[1]];
+                auto geo = self.GetGeometry();
+                geo->ProjectPointEdge
+                  (0,0,p1,
+                   const_cast<EdgePointGeomInfo*>(&el.epgeominfo[0]));
+                geo->ProjectPointEdge
+                  (0,0,p2,
+                   const_cast<EdgePointGeomInfo*>(&el.epgeominfo[1]));
+              }
             return self.AddSegment (el);
-          })
+          }, py::arg("el"), py::arg("project_geominfo")=false)
           
     .def ("Add", [](Mesh & self, const Element0d & el)
           {
@@ -1043,7 +1112,8 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                   ptr += 3;
                 }
           })
-    .def ("AddElements", [](Mesh & self, int dim, int index, py::buffer b1, int base)
+    .def ("AddElements", [](Mesh & self, int dim, int index, py::buffer b1, int base,
+                            bool project_geometry)
           {
             static Timer timer("Mesh::AddElements");
             static Timer timercast("Mesh::AddElements casting");
@@ -1095,6 +1165,21 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                     for (int j = 0; j < np; j++)
                       el[j] = ptr[j]+PointIndex::BASE-base;
                     el.SetIndex(index);
+                    if(project_geometry)
+                      {
+                        // find some point in the mid of trig/quad for
+                        // quick + stable uv-projection of all points
+                        auto startp = Center(self[el[0]], self[el[1]], self[el[2]]);
+                        PointGeomInfo gi = self.GetGeometry()->ProjectPoint(index,
+                                                                            startp);
+                        for(auto i : Range(np))
+                          {
+                            el.GeomInfo()[i] = gi;
+                            self.GetGeometry()->ProjectPointGI(index,
+                                                               self[el[i]],
+                                                               el.GeomInfo()[i]);
+                          }
+                      }
                     self.AddSurfaceElement (el);
                     ptr += info.strides[0]/sizeof(int);
                   }
@@ -1126,7 +1211,8 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                   }
               }
             
-          }, py::arg("dim"), py::arg("index"), py::arg("data"), py::arg("base")=0)
+          }, py::arg("dim"), py::arg("index"), py::arg("data"), py::arg("base")=0,
+          py::arg("project_geometry")=false)
     
     .def ("DeleteSurfaceElement",
           [](Mesh & self, SurfaceElementIndex i)
@@ -1164,7 +1250,7 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
             else
               throw Exception("either 'dim' or 'codim' must be specified");
             
-            NgArray<string*> & codimnames = self.GetRegionNamesCD (codim);
+            Array<string*> & codimnames = self.GetRegionNamesCD (codim);
             
             std::vector<string> names;
             for (auto name : codimnames)
@@ -1196,7 +1282,10 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            py::list points;
            for(const auto& pair : self.GetIdentifications().GetIdentifiedPoints())
              {
-               py::tuple pnts = py::make_tuple(pair.first.I1(), pair.first.I2());
+               // py::tuple pnts = py::make_tuple(pair.first.I1(), pair.first.I2());
+               
+               auto [pi1, pi2] = get<0> (pair.first);
+               py::tuple pnts = py::make_tuple(pi1, pi2);
                points.append(pnts);
              }
            return points;
@@ -1215,7 +1304,12 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            py::arg("identnr"),
            py::arg("type")=Identifications::PERIODIC)
     .def("IdentifyPeriodicBoundaries", &Mesh::IdentifyPeriodicBoundaries,
-         py::arg("face1"), py::arg("face2"), py::arg("mapping"), py::arg("point_tolerance") = -1.)
+         py::arg("identification_name"), py::arg("face1"), py::arg("mapping"),
+py::arg("point_tolerance") = -1.)
+    .def("GetCurveOrder", [] (Mesh & self)
+          {
+            return self.GetCurvedElements().GetOrder();
+          })
     .def("GetNrIdentifications", [](Mesh& self)
                                  {
                                    return self.GetIdentifications().GetMaxNr();
@@ -1234,15 +1328,12 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            {
              MeshingParameters mp;
              if(pars) mp = *pars;
-             {
-               py::gil_scoped_acquire acquire;
-               CreateMPfromKwargs(mp, kwargs);
-             }
+             CreateMPfromKwargs(mp, kwargs);
+             py::gil_scoped_release gil_release;
              MeshVolume (mp, self);
              OptimizeVolume (mp, self);
            }, py::arg("mp")=nullptr,
-          meshingparameter_description.c_str(),
-          py::call_guard<py::gil_scoped_release>())
+          meshingparameter_description.c_str())
 
     .def ("OptimizeVolumeMesh", [](Mesh & self, MeshingParameters* pars)
           {
@@ -1399,140 +1490,37 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            }))
     */
     
-    .def ("BuildSearchTree", &Mesh::BuildElementSearchTree,py::call_guard<py::gil_scoped_release>())
+    .def ("BuildSearchTree", &Mesh::BuildElementSearchTree,py::call_guard<py::gil_scoped_release>(),
+          py::arg("dim")=3)
 
     .def ("BoundaryLayer2", GenerateBoundaryLayer2, py::arg("domain"), py::arg("thicknesses"), py::arg("make_new_domain")=true, py::arg("boundaries")=Array<int>{})
-    .def ("BoundaryLayer", [](Mesh & self, variant<string, int> boundary,
-                              variant<double, py::list> thickness,
-                              variant<string, map<string, string>> material,
-                              variant<string, int> domain, bool outside,
-                              optional<string> project_boundaries,
+    .def ("BoundaryLayer", [](Mesh & self, variant<string, int, std::vector<int>> boundary,
+                              variant<double, std::vector<double>> thickness,
+                              optional<variant<string, map<string, string>>> material,
+                              variant<string, int, std::vector<int>> domain, bool outside,
+                              optional<variant<string, std::vector<int>>> project_boundaries,
                               bool grow_edges, bool limit_growth_vectors,
                               bool sides_keep_surfaceindex,
-                              bool keep_surfaceindex)
+                              bool disable_curving)
            {
+             throw Exception("Call syntax has changed! Pass a list of BoundaryLayerParameters to the GenerateMesh call instead: \ngeo.GenerateMesh(..., boundary_layers=[BoundaryLayerParameters(...), BoundaryLayerParameters(...), ...])");
              BoundaryLayerParameters blp;
-             BitArray boundaries(self.GetNFD()+1);
-             boundaries.Clear();
-             if(int* bc = get_if<int>(&boundary); bc)
-               {
-                 for (int i = 1; i <= self.GetNFD(); i++)
-                   if(self.GetFaceDescriptor(i).BCProperty() == *bc)
-                     boundaries.SetBit(i);
-               }
-             else
-               {
-                 regex pattern(*get_if<string>(&boundary));
-                 for(int i = 1; i<=self.GetNFD(); i++)
-                   {
-                     auto& fd = self.GetFaceDescriptor(i);
-                     if(regex_match(fd.GetBCName(), pattern))
-                       {
-                         boundaries.SetBit(i);
-                         auto dom_pattern = get_if<string>(&domain);
-                         // only add if adjacent to domain
-                         if(dom_pattern)
-                           {
-                             regex pattern(*dom_pattern);
-                             bool mat1_match = fd.DomainIn() > 0 && regex_match(self.GetMaterial(fd.DomainIn()), pattern);
-                             bool mat2_match = fd.DomainOut() > 0 && regex_match(self.GetMaterial(fd.DomainOut()), pattern);
-                             // if boundary is inner or outer remove from list
-                             if(mat1_match == mat2_match)
-                               boundaries.Clear(i);
-                             // if((fd.DomainIn() > 0 && regex_match(self.GetMaterial(fd.DomainIn()), pattern)) || (fd.DomainOut() > 0 && regex_match(self.GetMaterial(fd.DomainOut()), pattern)))
-                             // boundaries.Clear(i);
-                             // blp.surfid.Append(i);
-                           }
-                         // else
-                         //   blp.surfid.Append(i);
-                       }
-                     }
-               }
-             for(int i = 1; i<=self.GetNFD(); i++)
-               if(boundaries.Test(i))
-                 blp.surfid.Append(i);
-             if(string* mat = get_if<string>(&material); mat)
-                 blp.new_mat = { { ".*", *mat } };
-             else
-                 blp.new_mat = *get_if<map<string, string>>(&material);
-
-             if(project_boundaries.has_value())
-               {
-                 regex pattern(*project_boundaries);
-                 for(int i = 1; i<=self.GetNFD(); i++)
-                   if(regex_match(self.GetFaceDescriptor(i).GetBCName(), pattern))
-                     blp.project_boundaries.Append(i);
-               }
-
-             if(double* pthickness = get_if<double>(&thickness); pthickness)
-               {
-                 blp.heights.Append(*pthickness);
-               }
-             else
-               {
-                 auto thicknesses = *get_if<py::list>(&thickness);
-                 for(auto val : thicknesses)
-                   blp.heights.Append(val.cast<double>());
-               }
-
-             int nr_domains = self.GetNDomains();
-             blp.domains.SetSize(nr_domains + 1); // one based
-             blp.domains.Clear();
-             if(string* pdomain = get_if<string>(&domain); pdomain)
-               {
-                 regex pattern(*pdomain);
-                 for(auto i : Range(1, nr_domains+1))
-                   if(regex_match(self.GetMaterial(i), pattern))
-                     blp.domains.SetBit(i);
-               }
-             else
-               {
-                 auto idomain = *get_if<int>(&domain);
-                 blp.domains.SetBit(idomain);
-               }
-
+             blp.boundary = boundary;
+             blp.thickness = thickness;
+             blp.new_material = material;
+             blp.domain = domain;
              blp.outside = outside;
+             blp.project_boundaries = project_boundaries;
              blp.grow_edges = grow_edges;
              blp.limit_growth_vectors = limit_growth_vectors;
              blp.sides_keep_surfaceindex = sides_keep_surfaceindex;
-             blp.keep_surfaceindex = keep_surfaceindex;
-
+             blp.disable_curving = disable_curving;
              GenerateBoundaryLayer (self, blp);
              self.UpdateTopology();
-           }, py::arg("boundary"), py::arg("thickness"), py::arg("material"),
+           }, py::arg("boundary"), py::arg("thickness"), py::arg("material")=nullopt,
           py::arg("domains") = ".*", py::arg("outside") = false,
-          py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true, py::arg("limit_growth_vectors") = true, py::arg("sides_keep_surfaceindex")=false,
-          py::arg("keep_surfaceindex")=false,
-          R"delimiter(
-Add boundary layer to mesh.
-
-Parameters
-----------
-
-boundary : string or int
-  Boundary name or number.
-
-thickness : float or List[float]
-  Thickness of boundary layer(s).
-
-material : str or List[str]
-  Material name of boundary layer(s).
-
-domain : str or int
-  Regexp for domain boundarylayer is going into.
-
-outside : bool = False
-  If true add the layer on the outside
-
-grow_edges : bool = False
-  Grow boundary layer over edges.
-
-project_boundaries : Optional[str] = None
-  Project boundarylayer to these boundaries if they meet them. Set
-  to boundaries that meet boundarylayer at a non-orthogonal edge and
-  layer-ending should be projected to that boundary.
-
-)delimiter")
+          py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true, py::arg("limit_growth_vectors") = false, py::arg("sides_keep_surfaceindex")=false,
+          py::arg("disable_curving")=true, "Add boundary layer to mesh. see help(BoundaryLayerParameters) for details.")
 
     .def_static ("EnableTableClass", [] (string name, bool set)
           {
@@ -1598,7 +1586,7 @@ project_boundaries : Optional[str] = None
                 {
                     const auto & seg = segs[i];
                     for(auto k : Range(2))
-                        output[2*i+k] = seg[k]-PointIndex::BASE;
+                      output[2*i+k] = seg[k]-IndexBASE<PointIndex>();
                 } });
             return output;
           })
@@ -1617,8 +1605,8 @@ project_boundaries : Optional[str] = None
                   // PointIndex p0,p1;
                   // topo.GetEdgeVertices(i+1, p0, p1);
                   auto [p0,p1] = topo.GetEdgeVertices(i);
-                    output[2*i] = p0-PointIndex::BASE;
-                    output[2*i+1] = p1-PointIndex::BASE;
+                    output[2*i] = p0-IndexBASE<PointIndex>();
+                    output[2*i+1] = p1-IndexBASE<PointIndex>();
                 } });
             return output;
           })
@@ -1636,7 +1624,7 @@ project_boundaries : Optional[str] = None
                     const auto & sel = surfels[i];
                     auto * trig = &trigs[3*i];
                     for(auto k : Range(3))
-                        trig[k] = sel[k]-PointIndex::BASE;
+                        trig[k] = sel[k]-IndexBASE<PointIndex>();
                         // todo: quads (store the second trig in thread-local extra array, merge them at the end (mutex)
                 } });
             return trigs;
@@ -1654,7 +1642,7 @@ project_boundaries : Optional[str] = None
                     const auto & el = els[i];
                     auto * trig = &tets[4*i];
                     for(auto k : Range(4))
-                        trig[k] = el[k]-PointIndex::BASE;
+                        trig[k] = el[k]-IndexBASE<PointIndex>();
                         // todo: prisms etc (store the extra tets in thread-local extra array, merge them at the end (mutex)
                 } });
             return tets;
@@ -1705,25 +1693,25 @@ project_boundaries : Optional[str] = None
                     return mp;
                   }), py::arg("mp")=nullptr, meshingparameter_description.c_str())
     .def("__str__", &ToString<MP>)
-    .def("RestrictH", [](MP & mp, double x, double y, double z, double h)
+    .def("RestrictH", [](MP & mp, double x, double y, double z, double h, int layer)
           {
-            mp.meshsize_points.Append ( MeshingParameters::MeshSizePoint(Point<3> (x,y,z), h));
-          }, py::arg("x"), py::arg("y"), py::arg("z"), py::arg("h")
+            mp.meshsize_points.Append ( MeshingParameters::MeshSizePoint(Point<3> (x,y,z), h, layer));
+          }, py::arg("x"), py::arg("y"), py::arg("z"), py::arg("h"), py::arg("layer")=1
          )
-    .def("RestrictH", [](MP & mp, const Point<3>& p, double h)
+    .def("RestrictH", [](MP & mp, const Point<3>& p, double h, int layer)
     {
-      mp.meshsize_points.Append ({p, h});
-    }, py::arg("p"), py::arg("h"))
+      mp.meshsize_points.Append ({p, h, layer});
+    }, py::arg("p"), py::arg("h"), py::arg("layer")=1)
     .def("RestrictHLine", [](MP& mp, const Point<3>& p1, const Point<3>& p2,
-                             double maxh)
+                             double maxh, int layer)
     {
       int steps = int(Dist(p1, p2) / maxh) + 2;
       auto v = p2 - p1;
       for (int i = 0; i <= steps; i++)
         {
-          mp.meshsize_points.Append({p1 + double(i)/steps * v, maxh});
+          mp.meshsize_points.Append({p1 + double(i)/steps * v, maxh, layer});
         }
-    }, py::arg("p1"), py::arg("p2"), py::arg("maxh"))
+    }, py::arg("p1"), py::arg("p2"), py::arg("maxh"), py::arg("layer")=1)
     ;
 
   m.def("SetTestoutFile", FunctionPointer ([] (const string & filename)
@@ -1758,6 +1746,92 @@ project_boundaries : Optional[str] = None
 
   m.attr("debugparam") = py::cast(&debugparam);
 
+  py::class_<BoundaryLayerParameters>(m, "BoundaryLayerParameters")
+    .def(py::init([]( 
+        std::variant<string, int, std::vector<int>> boundary,
+        std::variant<double, std::vector<double>> thickness,
+        std::optional<std::variant<string, std::map<string, string>>> new_material,
+        std::variant<string, int, std::vector<int>> domain,
+        bool outside,
+        std::optional<std::variant<string, std::vector<int>>> project_boundaries,
+        bool grow_edges,
+        bool limit_growth_vectors,
+        std::optional<bool> sides_keep_surfaceindex,
+        bool disable_curving)
+        {
+          BoundaryLayerParameters blp;
+          blp.boundary = boundary;
+          blp.thickness = thickness;
+          blp.new_material = new_material;
+          blp.domain = domain;
+          blp.outside = outside;
+          blp.project_boundaries = project_boundaries;
+          blp.grow_edges = grow_edges;
+          blp.limit_growth_vectors = limit_growth_vectors;
+          blp.sides_keep_surfaceindex = sides_keep_surfaceindex;
+          blp.disable_curving = disable_curving;
+          return blp;
+        }),
+           py::arg("boundary"), py::arg("thickness"), py::arg("new_material")=nullopt,
+           py::arg("domain") = ".*", py::arg("outside") = false,
+           py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true,
+           py::arg("limit_growth_vectors") = false, py::arg("sides_keep_surfaceindex")=nullopt,
+           py::arg("disable_curving")=true,
+           R"delimiter(
+Add boundary layer to mesh.
+
+Parameters
+----------
+
+boundary : string or int
+  Boundary name or number.
+
+thickness : float or List[float]
+  Thickness of boundary layer(s).
+
+material : str or List[str]
+  Material name of boundary layer(s).
+
+domain : str or int
+  Regexp for domain boundarylayer is going into.
+
+outside : bool = False
+  If true add the layer on the outside
+
+grow_edges : bool = False
+  Grow boundary layer over edges.
+
+project_boundaries : Optional[str] = None
+  Project boundarylayer to these boundaries if they meet them. Set
+  to boundaries that meet boundarylayer at a non-orthogonal edge and
+  layer-ending should be projected to that boundary.
+
+)delimiter")
+    .def(py::init([]( const py::dict & d ) {
+      try {
+        // Call other constructor with named arguments by unpacking the dictionary
+        py::object cls = py::type::of<BoundaryLayerParameters>();
+        return cls(**d).cast<BoundaryLayerParameters>();
+      }
+      catch (py::error_already_set & e) {
+        cerr << "Error creating BoundaryLayerParameters from dict:" << endl;
+        cerr << e.what() << endl;
+        throw;
+      }
+    }))
+    .def_readwrite("boundary", &BoundaryLayerParameters::boundary)
+    .def_readwrite("thickness", &BoundaryLayerParameters::thickness)
+    .def_readwrite("new_material", &BoundaryLayerParameters::new_material)
+    .def_readwrite("domain", &BoundaryLayerParameters::domain)
+    .def_readwrite("outside", &BoundaryLayerParameters::outside)
+    .def_readwrite("project_boundaries", &BoundaryLayerParameters::project_boundaries)
+    .def_readwrite("grow_edges", &BoundaryLayerParameters::grow_edges)
+    .def_readwrite("limit_growth_vectors", &BoundaryLayerParameters::limit_growth_vectors)
+    .def_readwrite("sides_keep_surfaceindex", &BoundaryLayerParameters::sides_keep_surfaceindex)
+    .def_readwrite("disable_curving", &BoundaryLayerParameters::disable_curving)
+    ;
+  py::implicitly_convertible<py::dict, BoundaryLayerParameters>();
+
 #ifdef NG_CGNS
   m.def("ReadCGNSFile", &ReadCGNSFile, py::arg("filename"), py::arg("base")=1, "Read mesh and solution vectors from CGNS file");
   m.def("WriteCGNSFile", &WriteCGNSFile, py::arg("mesh"), py::arg("filename"), py::arg("names"), py::arg("values"), py::arg("locations"),
