@@ -14,6 +14,7 @@
 #include "occgeom.hpp"
 #include "Partition_Spliter.hxx"
 
+#include <BinTools.hxx>
 #include <BOPAlgo_Builder.hxx>
 #include <BRepBndLib.hxx>
 #include <BRepBuilderAPI_Copy.hxx>
@@ -199,6 +200,11 @@ namespace netgen
       return *faces[fmap.FindIndex(shape)-1];
   }
 
+  const GeometrySolid & OCCGeometry :: GetSolid(const TopoDS_Shape & shape) const
+  {
+      return *solids[somap.FindIndex(shape)-1];
+  }
+
 
   string STEP_GetEntityName(const TopoDS_Shape & theShape, STEPCAFControl_Reader * aReader)
   {
@@ -415,16 +421,7 @@ namespace netgen
       }
 #endif
 
-#ifdef OCC_HAVE_HISTORY    
-    Handle(BRepTools_History) history = aBuilder.History ();
-    
-    for (TopExp_Explorer e(shape, TopAbs_SOLID); e.More(); e.Next())
-      {
-        if (auto name = OCCGeometry::GetProperties(e.Current()).name)
-          for (auto mods : history->Modified(e.Current()))
-            OCCGeometry::GetProperties(mods).name = *name;
-      }
-#endif // OCC_HAVE_HISTORY    
+    PropagateProperties(aBuilder, shape);
     
     // result of the operation
     shape = aBuilder.Shape();
@@ -926,7 +923,7 @@ namespace netgen
          // Need do copy the face, otherwise replace is ignored
          BRepBuilderAPI_Copy copy(face);
          auto newface = copy.Shape().Reversed();
-         GetProperties(newface).Merge(GetProperties(face));
+         PropagateProperties(copy, face);
          rebuild->Replace(face, newface);
        }
      }
@@ -1135,6 +1132,21 @@ namespace netgen
          }
       }
       */
+
+      std::map<int, ArrayMem<int, 10>> free_edges_in_solid;
+      for(auto i1 : Range(1, somap.Extent()+1))
+      {
+          auto s = somap(i1);
+          for (auto edge : MyExplorer(s, TopAbs_EDGE, TopAbs_WIRE))
+            if (!emap.Contains(edge))
+              {
+                free_edges_in_solid[i1].Append(emap.Add (edge));
+                for (auto vertex : MyExplorer(edge, TopAbs_VERTEX))
+                  if (!vmap.Contains(vertex))
+                    vmap.Add (vertex);
+              }
+      }
+
       for (auto edge : MyExplorer(shape, TopAbs_EDGE, TopAbs_WIRE))
         if (!emap.Contains(edge))
           {
@@ -1210,7 +1222,8 @@ namespace netgen
           if(verts.size() == 0)
             continue;
           auto occ_edge = make_unique<OCCEdge>(edge, GetVertex(verts[0]), GetVertex(verts[1]) );
-          occ_edge->properties = GetProperties(e);
+          if(HaveProperties(edge))
+            occ_edge->properties = GetProperties(e);
           edges.Append(std::move(occ_edge));
       }
 
@@ -1247,11 +1260,11 @@ namespace netgen
           auto occ_solid = make_unique<OCCSolid>(s);
           if(HaveProperties(s))
             occ_solid->properties = GetProperties(s);
-          solids.Append(std::move(occ_solid));
-
           for(auto f : GetFaces(s))
           {
               auto & face = static_cast<OCCFace&>(GetFace(f));
+              face.properties.maxh = min2(face.properties.maxh, occ_solid->properties.maxh);
+
               if(face.domin==-1)
                   face.domin = k;
               else
@@ -1259,21 +1272,20 @@ namespace netgen
               if(face.Shape().Orientation() == TopAbs_INTERNAL)
                   face.domout = k;
           }
+
+          if(free_edges_in_solid.count(i1))
+            for(auto ei : free_edges_in_solid[i1])
+            {
+              auto & edge = GetEdge(emap(ei));
+              edge.properties.maxh = min(edge.properties.maxh, occ_solid->properties.maxh);
+              edge.domin = k;
+              edge.domout = k;
+              occ_solid->free_edges.Append(&GetEdge(emap(ei)));
+            }
+          solids.Append(std::move(occ_solid));
       }
 
       // Propagate maxh to children
-      for(auto& solid : solids)
-      {
-        auto& shape = static_cast<OCCSolid&>(*solid).GetShape();
-        if(!OCCGeometry::HaveProperties(shape))
-          continue;
-        for(auto& f : GetFaces(shape))
-        {
-          auto& face = GetFace(f);
-          face.properties.maxh = min2(face.properties.maxh,
-                                      GetProperties(shape).maxh);
-        }
-      }
       for(auto& face : faces)
         for(auto& edge : face->edges)
           edge->properties.maxh = min2(edge->properties.maxh,
@@ -1638,8 +1650,12 @@ namespace netgen
 
       if(!result)
       {
-         delete occgeo;
-         return NULL;
+        result = BinTools::Read(occgeo->shape, filename.string().c_str());
+        if (!result)
+          {
+            delete occgeo;
+            throw Exception("Could not read BREP file " + filename.string());
+          }
       }
 
       occgeo->changed = 1;
@@ -1732,7 +1748,10 @@ namespace netgen
     Array<TopoDS_Shape> shape_list;
 
     ar & dimension;
-    for (auto typ : { TopAbs_SOLID, TopAbs_FACE,  TopAbs_EDGE })
+    auto types = Array<TopAbs_ShapeEnum>{ TopAbs_SOLID, TopAbs_FACE,  TopAbs_EDGE };
+    if(ar.GetVersion("netgen") >= "v6.2.2406-22")
+      types.Append(TopAbs_VERTEX);
+    for (auto typ : types)
       for (TopExp_Explorer e(shape, typ); e.More(); e.Next())
         {
           auto ds = e.Current();
@@ -1754,8 +1773,19 @@ namespace netgen
         ar & has_identifications;
         if(has_identifications)
           {
-            auto & idents = GetIdentifications(s);
-            auto n_idents = idents.size();
+            int n_idents;
+            std::vector<OCCIdentification> used_idents;
+            if(ar.Output())
+              {
+                // only use identifications that are used within the geometry
+                for(auto& id : GetIdentifications(s))
+                  {
+                    if(shape_map.Contains(id.from) && shape_map.Contains(id.to))
+                      used_idents.push_back(id);
+                  }
+                n_idents = used_idents.size();
+              }
+            auto & idents = ar.Output() ? used_idents : GetIdentifications(s);
             ar & n_idents;
             idents.resize(n_idents);
             for(auto i : Range(n_idents))
@@ -1767,7 +1797,18 @@ namespace netgen
                   id_from = shape_map.FindIndex(id.from)-1;
                   id_to = shape_map.FindIndex(id.to)-1;
                 }
-                ar & id_from & id_to & id.trafo & id.name;
+                ar & id_from & id_to;
+
+                // trafo is now optional -> special treatment necessary for backward compatibility
+                if(ar.Output() || netgen_version >= "v6.2.2403-34-g571cbbe4")
+                  ar & id.trafo;
+                else
+                {
+                  auto trafo = Transformation<3>();
+                  ar & trafo;
+                  id.trafo = trafo;
+                }
+                ar & id.name;
                 if(ar.Input())
                 {
                     id.from = shape_list[id_from];
@@ -2248,10 +2289,22 @@ namespace netgen
             XCAFPrs::CollectStyleSettings(label, loc, set);
             XCAFPrs_Style aStyle;
             set.FindFromKey(e.Current(), aStyle);
-
-            auto & prop = OCCGeometry::GetProperties(e.Current());
             if(aStyle.IsSetColorSurf())
-                prop.col = step_utils::ReadColor(aStyle.GetColorSurfRGBA());
+              {
+                for(TopExp_Explorer e2(e.Current(), TopAbs_FACE); e2.More(); e2.Next())
+                  {
+                    auto & prop = OCCGeometry::GetProperties(e2.Current());
+                    prop.col = step_utils::ReadColor(aStyle.GetColorSurfRGBA());
+                  }
+              }
+            if(aStyle.IsSetColorCurv())
+              {
+                for(TopExp_Explorer e2(e.Current(), TopAbs_EDGE); e2.More(); e2.Next())
+                  {
+                    auto & prop = OCCGeometry::GetProperties(e2.Current());
+                    prop.col = step_utils::ReadColor(aStyle.GetColorSurfRGBA());
+                  }
+              }
           }
 
         // load names
@@ -2266,10 +2319,14 @@ namespace netgen
 
             TopoDS_Shape shape = TransferBRep::ShapeResult(transProc->Find(item));
             string name = item->Name()->ToCString();
-            if (!transProc->IsBound(item))
+            if (!transProc->IsBound(item) || name == "")
               continue;
 
-            OCCGeometry::GetProperties(shape).name = name;
+            // we only allow names on SOLIDS, FACES, EDGES, VERTICES.
+            // if name is given on a compound, assume it should be on all subshapes
+            // of highest dimension
+            for(auto & s : GetHighestDimShapes(shape))
+              OCCGeometry::GetProperties(s).name = name;
           }
 
 
