@@ -25,7 +25,7 @@ DLL_HEADER void ExportNgOCCBasic(py::module &m)
     .def(py::init([] (py::tuple pnt)
                   {
                     if (py::len(pnt) != 3)
-                      throw Exception("need 3-tuple to create gp_Pnt");
+                      throw std::length_error("need 3-tuple to create gp_Pnt");
                     
                     return gp_Pnt(py::cast<double>(pnt[0]),
                                   py::cast<double>(pnt[1]),
@@ -73,6 +73,7 @@ DLL_HEADER void ExportNgOCCBasic(py::module &m)
     .def(py::init([] (double x, double y, double z) {
           return gp_Vec(x, y, z);
         }), py::arg("x"), py::arg("y"), py::arg("z"))
+    .def(py::init([](gp_Dir d) { return gp_Vec(d); }))
     .def_property("x", [](gp_Vec&p) { return p.X(); }, [](gp_Vec&p,double x) { p.SetX(x); })
     .def_property("y", [](gp_Vec&p) { return p.Y(); }, [](gp_Vec&p,double y) { p.SetY(y); })
     .def_property("z", [](gp_Vec&p) { return p.Z(); }, [](gp_Vec&p,double z) { p.SetZ(z); })
@@ -131,6 +132,16 @@ DLL_HEADER void ExportNgOCCBasic(py::module &m)
         str << "(" << p.X() << ", " << p.Y() << ", " << p.Z() << ")";
         return str.str();
       })
+    .def("__getitem__", [](const gp_Dir& d, int index)
+    {
+      if(index == 0)
+        return d.X();
+      if(index == 1)
+        return d.Y();
+      if(index == 2)
+        return d.Z();
+      throw std::out_of_range("Direction index must be in range [0,3)!");
+    })
     ;
 
 
@@ -166,12 +177,9 @@ DLL_HEADER void ExportNgOCCBasic(py::module &m)
 
 
   py::class_<gp_Pnt2d>(m, "gp_Pnt2d", "2d OCC point")
-    .def(py::init([] (py::tuple pnt)
+    .def(py::init([] (std::tuple<double,double> pnt)
                   {
-                    if (py::len(pnt) != 2)
-                      throw Exception("need 2-tuple to create gp_Pnt2d");
-                    return gp_Pnt2d(py::cast<double>(pnt[0]),
-                                    py::cast<double>(pnt[1]));
+                    return gp_Pnt2d(get<0>(pnt), get<1>(pnt));
                   }))
     .def(py::init([] (double x, double y) {
           return gp_Pnt2d(x, y);
@@ -368,7 +376,8 @@ DLL_HEADER void ExportNgOCCBasic(py::module &m)
   py::implicitly_convertible<py::tuple, gp_Pnt>();
   py::implicitly_convertible<py::tuple, gp_Vec>();
   py::implicitly_convertible<py::tuple, gp_Dir>();
-  py::implicitly_convertible<gp_Vec, gp_Dir>();  
+  py::implicitly_convertible<gp_Vec, gp_Dir>();
+  py::implicitly_convertible<gp_Dir, gp_Vec>();
   py::implicitly_convertible<py::tuple, gp_Pnt2d>();  
   py::implicitly_convertible<py::tuple, gp_Vec2d>();  
   py::implicitly_convertible<py::tuple, gp_Dir2d>();
